﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ComicReader
{
    /// <summary>
    /// Selected Win AI Function Calls
    /// </summary>
    public class WinApi
    {
        [DllImport("user32.dll", EntryPoint = "GetSystemMetrics")]
        public static extern int GetSystemMetrics(int which);

        [DllImport("user32.dll")]
        public static extern void
            SetWindowPos(IntPtr hwnd, IntPtr hwndInsertAfter,
                         int X, int Y, int width, int height, uint flags);

        private const int SM_CXSCREEN = 0;
        private const int SM_CYSCREEN = 1;
        private static IntPtr HWND_TOP = IntPtr.Zero;
        private const int SWP_SHOWWINDOW = 64; // 0x0040

        public static int ScreenX
        {
            get { return GetSystemMetrics(SM_CXSCREEN); }
        }

        public static int ScreenY
        {
            get { return GetSystemMetrics(SM_CYSCREEN); }
        }

        public static void SetWinFullScreen(IntPtr hwnd)
        {
            SetWindowPos(hwnd, HWND_TOP, 0, 0, ScreenX, ScreenY, SWP_SHOWWINDOW);
        }
    }

    /// <summary>
    /// Class used to preserve / restore state of the form
    /// </summary>
    public class FormState
    {
        private FormWindowState winState;
        private FormBorderStyle brdStyle;
        ////////////
        private int height, width;
        private Point point;
        ////////////
        private bool IsMaximized = false;

        public void Maximize(MainForm targetForm) // Form -> MainForm
        {
            if (!IsMaximized)
            {
                IsMaximized = true;
                Save(targetForm);
                targetForm.WindowState = FormWindowState.Maximized;
                targetForm.FormBorderStyle = FormBorderStyle.None;
                //////////
                targetForm.menuStrip.Enabled = false;
                targetForm.pictureBox.Location = new Point(0, 0);
                targetForm.pictureBox.Height = WinApi.ScreenY;
                targetForm.pictureBox.Width = WinApi.ScreenX + 17;
                //////////
                WinApi.SetWinFullScreen(targetForm.Handle);
            }
            else
                Restore(targetForm);
        }

        public void Save(MainForm targetForm) // Form -> MainForm
        {
            winState = targetForm.WindowState;
            brdStyle = targetForm.FormBorderStyle;
            /////////
            height = targetForm.pictureBox.Height;
            width = targetForm.pictureBox.Width;
            point = targetForm.pictureBox.Location;
            /////////
        }

        public void Restore(MainForm targetForm) // Form -> MainForm
        {
            targetForm.WindowState = winState;
            targetForm.FormBorderStyle = brdStyle;
            //////////
            targetForm.menuStrip.Enabled = true;
            targetForm.pictureBox.Height = height;
            targetForm.pictureBox.Width = width;
            targetForm.pictureBox.Location = point;
            //////////
            IsMaximized = false;
        }
    }
}